==> ansible-12.1.0-1-any.pkg.tar.zst: usr/lib/python3.13/site-packages/ansible_collections/cisco/meraki/plugins/action/networks_meraki_auth_users.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.meraki.plugins.plugin_utils.meraki import (
    MERAKI,
    meraki_argument_spec,
    meraki_compare_equality2,
    get_dict_result,
)
from ansible_collections.cisco.meraki.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = meraki_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present", "absent"]),
    accountType=dict(type="str"),
    authorizations=dict(type="list"),
    email=dict(type="str"),
    emailPasswordToUser=dict(type="bool"),
    isAdmin=dict(type="bool"),
    name=dict(type="str"),
    password=dict(type="str"),
    networkId=dict(type="str"),
    merakiAuthUserId=dict(type="str"),
    delete=dict(type="bool"),
))

required_if = [
    ("state", "present", ["merakiAuthUserId", "name", "networkId"], True),
    ("state", "absent", ["merakiAuthUserId", "name", "networkId"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class NetworksMerakiAuthUsers(object):
    def __init__(self, params, meraki):
        self.meraki = meraki
        self.new_object = dict(
            accountType=params.get("accountType"),
            authorizations=params.get("authorizations"),
            email=params.get("email"),
            emailPasswordToUser=params.get("emailPasswordToUser"),
            isAdmin=params.get("isAdmin"),
            name=params.get("name"),
            password=params.get("password"),
            networkId=params.get("networkId"),
            merakiAuthUserId=params.get("merakiAuthUserId"),
            delete=params.get("delete"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get('network_id') is not None:
            new_object_params['networkId'] = self.new_object.get('networkId') or \
                self.new_object.get('network_id')
        return new_object_params

    def get_params_by_id(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get('network_id') is not None:
            new_object_params['networkId'] = self.new_object.get('networkId') or \
                self.new_object.get('network_id')
        if self.new_object.get('merakiAuthUserId') is not None or self.new_object.get('meraki_auth_user_id') is not None:
            new_object_params['merakiAuthUserId'] = self.new_object.get('merakiAuthUserId') or \
                self.new_object.get('meraki_auth_user_id')
        return new_object_params

    def create_params(self):
        new_object_params = {}
        if self.new_object.get('accountType') is not None or self.new_object.get('account_type') is not None:
            new_object_params['accountType'] = self.new_object.get('accountType') or \
                self.new_object.get('account_type')
        if self.new_object.get('authorizations') is not None or self.new_object.get('authorizations') is not None:
            new_object_params['authorizations'] = self.new_object.get('authorizations') or \
                self.new_object.get('authorizations')
        if self.new_object.get('email') is not None or self.new_object.get('email') is not None:
            new_object_params['email'] = self.new_object.get('email') or \
                self.new_object.get('email')
        if self.new_object.get('emailPasswordToUser') is not None or self.new_object.get('email_password_to_user') is not None:
            new_object_params['emailPasswordToUser'] = self.new_object.get(
                'emailPasswordToUser')
        if self.new_object.get('isAdmin') is not None or self.new_object.get('is_admin') is not None:
            new_object_params['isAdmin'] = self.new_object.get('isAdmin')
        if self.new_object.get('name') is not None or self.new_object.get('name') is not None:
            new_object_params['name'] = self.new_object.get('name') or \
                self.new_object.get('name')
        if self.new_object.get('password') is not None or self.new_object.get('password') is not None:
            new_object_params['password'] = self.new_object.get('password') or \
                self.new_object.get('password')
        if self.new_object.get('networkId') is not None or self.new_object.get('network_id') is not None:
            new_object_params['networkId'] = self.new_object.get('networkId') or \
                self.new_object.get('network_id')
        return new_object_params

    def delete_by_id_params(self):
        new_object_params = {}
        if self.new_object.get('delete') is not None or self.new_object.get('delete') is not None:
            new_object_params['delete'] = self.new_object.get('delete')
        if self.new_object.get('networkId') is not None or self.new_object.get('network_id') is not None:
            new_object_params['networkId'] = self.new_object.get('networkId') or \
                self.new_object.get('network_id')
        if self.new_object.get('merakiAuthUserId') is not None or self.new_object.get('meraki_auth_user_id') is not None:
            new_object_params['merakiAuthUserId'] = self.new_object.get('merakiAuthUserId') or \
                self.new_object.get('meraki_auth_user_id')
        return new_object_params

    def update_by_id_params(self):
        new_object_params = {}
        if self.new_object.get('authorizations') is not None or self.new_object.get('authorizations') is not None:
            new_object_params['authorizations'] = self.new_object.get('authorizations') or \
                self.new_object.get('authorizations')
        if self.new_object.get('emailPasswordToUser') is not None or self.new_object.get('email_password_to_user') is not None:
            new_object_params['emailPasswordToUser'] = self.new_object.get(
                'emailPasswordToUser')
        if self.new_object.get('name') is not None or self.new_object.get('name') is not None:
            new_object_params['name'] = self.new_object.get('name') or \
                self.new_object.get('name')
        if self.new_object.get('password') is not None or self.new_object.get('password') is not None:
            new_object_params['password'] = self.new_object.get('password') or \
                self.new_object.get('password')
        if self.new_object.get('networkId') is not None or self.new_object.get('network_id') is not None:
            new_object_params['networkId'] = self.new_object.get('networkId') or \
                self.new_object.get('network_id')
        if self.new_object.get('merakiAuthUserId') is not None or self.new_object.get('meraki_auth_user_id') is not None:
            new_object_params['merakiAuthUserId'] = self.new_object.get('merakiAuthUserId') or \
                self.new_object.get('meraki_auth_user_id')
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method or it is in another action
        try:
            items = self.meraki.exec_meraki(
                family="networks",
                function="getNetworkMerakiAuthUsers",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'name', name)
            if result is None:
                result = items
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        try:
            items = self.meraki.exec_meraki(
                family="networks",
                function="getNetworkMerakiAuthUser",
                params=self.get_params_by_id()
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'merakiAuthUserId', id)
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def exists(self):
        id_exists = False
        name_exists = False
        prev_obj = None
        o_id = self.new_object.get("id")
        o_id = o_id or self.new_object.get(
            "meraki_auth_user_id") or self.new_object.get("merakiAuthUserId")
        name = self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            _id = _id or prev_obj.get("merakiAuthUserId")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters(
                    "The 'id' and 'name' params don't refer to the same object")
            if _id:
                self.new_object.update(dict(id=_id))
                self.new_object.update(dict(merakiAuthUserId=_id))
            if _id:
                prev_obj = self.get_object_by_id(_id)
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("accountType", "accountType"),
            ("authorizations", "authorizations"),
            ("email", "email"),
            ("emailPasswordToUser", "emailPasswordToUser"),
            ("isAdmin", "isAdmin"),
            ("name", "name"),
            ("networkId", "networkId"),
            ("merakiAuthUserId", "merakiAuthUserId"),
            ("delete", "delete"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (DNAC) params
        # If any does not have eq params, it requires update
        return any(not meraki_compare_equality2(current_obj.get(meraki_param),
                                                requested_obj.get(ansible_param))
                   for (meraki_param, ansible_param) in obj_params)

    def create(self):
        result = self.meraki.exec_meraki(
            family="networks",
            function="createNetworkMerakiAuthUser",
            params=self.create_params(),
            op_modifies=True,
        )
        return result

    def update(self):
        id = self.new_object.get("id")
        id = id or self.new_object.get("merakiAuthUserId")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
                id_ = id_ or prev_obj_name.get("merakiAuthUserId")
            if id_:
                self.new_object.update(dict(merakiAuthUserId=id_))
        result = self.meraki.exec_meraki(
            family="networks",
            function="updateNetworkMerakiAuthUser",
            params=self.update_by_id_params(),
            op_modifies=True,
        )
        return result

    def delete(self):
        id = self.new_object.get("id")
        id = id or self.new_object.get("merakiAuthUserId")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
                id_ = id_ or prev_obj_name.get("merakiAuthUserId")
            if id_:
                self.new_object.update(dict(merakiAuthUserId=id_))
        result = self.meraki.exec_meraki(
            family="networks",
            function="deleteNetworkMerakiAuthUser",
            params=self.delete_by_id_params(),
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        meraki = MERAKI(self._task.args)
        obj = NetworksMerakiAuthUsers(self._task.args, meraki)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    meraki.object_updated()
                else:
                    response = prev_obj
                    meraki.object_already_present()
            else:
                response = obj.create()
                meraki.object_created()

        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                response = obj.delete()
                meraki.object_deleted()
            else:
                meraki.object_already_absent()

        self._result.update(dict(meraki_response=response))
        self._result.update(meraki.exit_json())
        return self._result

==> ansible-13.3.0-1-any.pkg.tar.zst: usr/lib/python3.14/site-packages/ansible_collections/cisco/meraki/plugins/action/networks_meraki_auth_users.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator, )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.meraki.plugins.plugin_utils.meraki import (
    MERAKI,
    meraki_argument_spec,
    meraki_compare_equality2,
    get_dict_result,
)
from ansible_collections.cisco.meraki.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = meraki_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present", "absent"]),
    email=dict(type="str"),
    name=dict(type="str"),
    password=dict(type="str"),
    accountType=dict(type="str"),
    emailPasswordToUser=dict(type="bool"),
    isAdmin=dict(type="bool"),
    authorizations=dict(type="list"),
    networkId=dict(type="str"),
    merakiAuthUserId=dict(type="str"),
    delete=dict(type="bool"),
))

required_if = [
    ("state", "present", ["merakiAuthUserId", "name", "networkId"], True),
    ("state", "absent", ["merakiAuthUserId", "name", "networkId"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class NetworksMerakiAuthUsers(object):
    def __init__(self, params, meraki):
        self.meraki = meraki
        self.new_object = dict(
            email=params.get("email"),
            name=params.get("name"),
            password=params.get("password"),
            accountType=params.get("accountType"),
            emailPasswordToUser=params.get("emailPasswordToUser"),
            isAdmin=params.get("isAdmin"),
            authorizations=params.get("authorizations"),
            networkId=params.get("networkId"),
            merakiAuthUserId=params.get("merakiAuthUserId"),
            delete=params.get("delete"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        return new_object_params

    def get_params_by_id(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('merakiAuthUserId') is not None or self.new_object.get(
                'meraki_auth_user_id') is not None:
            new_object_params['merakiAuthUserId'] = self.new_object.get(
                'merakiAuthUserId') or self.new_object.get('meraki_auth_user_id')
        return new_object_params

    def create_params(self):
        new_object_params = {}
        if self.new_object.get('email') is not None or self.new_object.get(
                'email') is not None:
            new_object_params['email'] = self.new_object.get('email') or \
                self.new_object.get('email')
        if self.new_object.get('name') is not None or self.new_object.get(
                'name') is not None:
            new_object_params['name'] = self.new_object.get('name') or \
                self.new_object.get('name')
        if self.new_object.get('password') is not None or self.new_object.get(
                'password') is not None:
            new_object_params['password'] = self.new_object.get(
                'password') or self.new_object.get('password')
        if self.new_object.get('accountType') is not None or self.new_object.get(
                'account_type') is not None:
            new_object_params['accountType'] = self.new_object.get(
                'accountType') or self.new_object.get('account_type')
        if self.new_object.get('emailPasswordToUser') is not None or self.new_object.get(
                'email_password_to_user') is not None:
            new_object_params['emailPasswordToUser'] = self.new_object.get(
                'emailPasswordToUser')
        if self.new_object.get('isAdmin') is not None or self.new_object.get(
                'is_admin') is not None:
            new_object_params['isAdmin'] = self.new_object.get('isAdmin')
        if self.new_object.get('authorizations') is not None or self.new_object.get(
                'authorizations') is not None:
            new_object_params['authorizations'] = self.new_object.get(
                'authorizations') or self.new_object.get('authorizations')
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        return new_object_params

    def delete_by_id_params(self):
        new_object_params = {}
        if self.new_object.get('delete') is not None or self.new_object.get(
                'delete') is not None:
            new_object_params['delete'] = self.new_object.get('delete')
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('merakiAuthUserId') is not None or self.new_object.get(
                'meraki_auth_user_id') is not None:
            new_object_params['merakiAuthUserId'] = self.new_object.get(
                'merakiAuthUserId') or self.new_object.get('meraki_auth_user_id')
        return new_object_params

    def update_by_id_params(self):
        new_object_params = {}
        if self.new_object.get('name') is not None or self.new_object.get(
                'name') is not None:
            new_object_params['name'] = self.new_object.get('name') or \
                self.new_object.get('name')
        if self.new_object.get('password') is not None or self.new_object.get(
                'password') is not None:
            new_object_params['password'] = self.new_object.get(
                'password') or self.new_object.get('password')
        if self.new_object.get('emailPasswordToUser') is not None or self.new_object.get(
                'email_password_to_user') is not None:
            new_object_params['emailPasswordToUser'] = self.new_object.get(
                'emailPasswordToUser')
        if self.new_object.get('authorizations') is not None or self.new_object.get(
                'authorizations') is not None:
            new_object_params['authorizations'] = self.new_object.get(
                'authorizations') or self.new_object.get('authorizations')
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('merakiAuthUserId') is not None or self.new_object.get(
                'meraki_auth_user_id') is not None:
            new_object_params['merakiAuthUserId'] = self.new_object.get(
                'merakiAuthUserId') or self.new_object.get('meraki_auth_user_id')
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method or it is in another action
        try:
            items = self.meraki.exec_meraki(
                family="networks",
                function="getNetworkMerakiAuthUsers",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'name', name)
            if result is None:
                result = items
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        try:
            items = self.meraki.exec_meraki(
                family="networks",
                function="getNetworkMerakiAuthUser",
                params=self.get_params_by_id()
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'merakiAuthUserId', id)
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def exists(self):
        id_exists = False
        name_exists = False
        prev_obj = None
        o_id = self.new_object.get("id")
        o_id = o_id or self.new_object.get(
            "meraki_auth_user_id") or self.new_object.get("merakiAuthUserId")
        name = self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            _id = _id or prev_obj.get("merakiAuthUserId")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters(
                    "The 'id' and 'name' params don't refer to the same object")
            if _id:
                self.new_object.update(dict(id=_id))
                self.new_object.update(dict(merakiAuthUserId=_id))
            if _id:
                prev_obj = self.get_object_by_id(_id)
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("email", "email"),
            ("name", "name"),
            ("accountType", "accountType"),
            ("emailPasswordToUser", "emailPasswordToUser"),
            ("isAdmin", "isAdmin"),
            ("authorizations", "authorizations"),
            ("networkId", "networkId"),
            ("merakiAuthUserId", "merakiAuthUserId"),
            ("delete", "delete"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (DNAC) params
        # If any does not have eq params, it requires update
        return any(
            not meraki_compare_equality2(
                current_obj.get(meraki_param),
                requested_obj.get(ansible_param)) for (
                meraki_param,
                ansible_param) in obj_params)

    def create(self):
        result = self.meraki.exec_meraki(
            family="networks",
            function="createNetworkMerakiAuthUser",
            params=self.create_params(),
            op_modifies=True,
        )
        return result

    def update(self):
        id = self.new_object.get("id")
        id = id or self.new_object.get("merakiAuthUserId")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
                id_ = id_ or prev_obj_name.get("merakiAuthUserId")
            if id_:
                self.new_object.update(dict(merakiAuthUserId=id_))
        result = self.meraki.exec_meraki(
            family="networks",
            function="updateNetworkMerakiAuthUser",
            params=self.update_by_id_params(),
            op_modifies=True,
        )
        return result

    def delete(self):
        id = self.new_object.get("id")
        id = id or self.new_object.get("merakiAuthUserId")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
                id_ = id_ or prev_obj_name.get("merakiAuthUserId")
            if id_:
                self.new_object.update(dict(merakiAuthUserId=id_))
        result = self.meraki.exec_meraki(
            family="networks",
            function="deleteNetworkMerakiAuthUser",
            params=self.delete_by_id_params(),
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        meraki = MERAKI(self._task.args)
        obj = NetworksMerakiAuthUsers(self._task.args, meraki)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    meraki.object_updated()
                else:
                    response = prev_obj
                    meraki.object_already_present()
            else:
                response = obj.create()
                meraki.object_created()

        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                response = obj.delete()
                meraki.object_deleted()
            else:
                meraki.object_already_absent()

        self._result.update(dict(meraki_response=response))
        self._result.update(meraki.exit_json())
        return self._result
